@@ -8,12 +8,11 @@ import re
 import fnmatch
 import ast
 import collections
-#import keyword
-from .compat.imports import builtins
-from .compat.environ import *
+
 import pkgutil
 
-from .decorators import _property, version, param
+from .compat import *
+from .decorators import property, version, param
 from .utils import String
 
 
@@ -21,7 +20,7 @@ class ReflectDecorator(object):
     """The information of each individual decorator on a given ReflectMethod will
     be wrapped in this class"""
 
-    @_property
+    @property(cached="_parents")
     def parents(self):
         ret = []
         decor = self.decorator
@@ -53,12 +52,7 @@ class ReflectDecorator(object):
 class ReflectMethod(object):
     """Reflects a method on a class"""
 
-    @property
-    def positionals(self):
-        """return True if this method accepts *args"""
-        return self.get_info().get("positionals", False)
-
-    @_property
+    @property(cached="_required_args")
     def required_args(self):
         """return how many *args are needed to call the method"""
         ret = []
@@ -106,17 +100,17 @@ class ReflectMethod(object):
         #pout.v(info, self.params)
         return list(filter(lambda x: x is not None, ret))
 
-    @_property
+    @property(cached="_name")
     def name(self):
         """return the method name (GET, POST)"""
         return self.method_name
 
-    @_property
+    @property(cached="_decorators")
     def decorators(self):
         decorators = self.reflect_class.decorators
         return decorators.get(self.method_name, [])
 
-    @_property
+    @property(cached="_desc")
     def desc(self):
         """return the description of this method"""
         doc = None
@@ -148,10 +142,18 @@ class ReflectMethod(object):
         info = self.reflect_class.get_info()
         return info[self.name][self.method_name]
 
+    def has_positionals(self):
+        """return True if this method accepts *args"""
+        return self.get_info().get("positionals", False)
+
+    def has_keywords(self):
+        """return True if this method accepts **kwargs"""
+        return self.get_info().get("keywords", False)
+
 
 class ReflectHTTPMethod(ReflectMethod):
     """This encompasses the http verbs like POST and GET"""
-    @_property
+    @property(cached="_version")
     def version(self):
         ret = ""
         for decor in self.decorators:
@@ -168,13 +170,13 @@ class ReflectHTTPMethod(ReflectMethod):
             headers['Accept'] = "{};version={}".format(self.reflect_class.cls.content_type, version)
         return headers
 
-    @_property
+    @property(cached="_name")
     def name(self):
         """return the method name (GET, POST)"""
         bits = self.method_name.split("_", 2)
         return bits[0]
 
-    @_property
+    @property(cached="_params")
     def params(self):
         """return information about the params that the given http option takes"""
         ret = {}
@@ -187,13 +189,17 @@ class ReflectHTTPMethod(ReflectMethod):
 
         return ret
 
+#     def positionals_count(self):
+#         for p in self.params:
+
+
 
 class ReflectClass(object):
     method_class = ReflectMethod
 
     decorator_class = ReflectDecorator
 
-    @_property
+    @property(cached="_methods")
     def methods(self):
         """
         return the supported http method options that this class supports
@@ -215,7 +221,7 @@ class ReflectClass(object):
                 ))
         return ret
 
-    @_property
+    @property(cached="_decorators")
     def decorators(self):
         """Get all the decorators of all the option methods in the class
 
@@ -242,12 +248,12 @@ class ReflectClass(object):
     def classpath(self):
         return "{}.{}".format(self.module_name, self.class_name)
 
-    @_property
+    @property(cached="_module")
     def module(self):
         return self.reflect_module.module
         #return ReflectModule(self.module_name).module
 
-    @_property
+    @property(cached="_desc")
     def desc(self):
         """return the description of this endpoint"""
         doc = inspect.getdoc(self.cls) or ""
@@ -262,6 +268,20 @@ class ReflectClass(object):
         """return True if this class is considered private"""
         return self.class_name.startswith('_')
 
+    def method(self, method_name):
+        """Returns information about the method_name on this class
+
+        :param method_name: string, the name of the method
+        :returns: ReflectMethod, the reflection information about the method
+        """
+        info = self.get_info()
+        for http_method, methods in info.items():
+            if method_name in methods:
+                method_info = methods[method_name]
+                return self.method_class(method_name, method_info["method"], reflect_class=self)
+
+        raise ValueError("No {}.{} method".format(self.classpath, method_name))
+
     def get_info(self):
         """Get all the decorators of all the option methods in the class
 
@@ -478,7 +498,7 @@ class ReflectController(ReflectClass):
 
     method_class = ReflectHTTPMethod
 
-    @_property
+    @property(cached="_bits")
     def bits(self):
         bits = self.module_name.replace(self.controller_prefix, '', 1).lower()
         bits = list(filter(None, bits.split(".")))
@@ -489,7 +509,7 @@ class ReflectController(ReflectClass):
 
         return bits
 
-    @_property
+    @property(cached="_uri")
     def uri(self):
         return "/" + "/".join(self.bits)
 
