@@ -458,225 +458,6 @@ class AuthTest(TestCase):
         r.body_kwargs = {"foo": "bar"}
         c.foo()
 
-#     def test_auth_child(self):
-# 
-#         class auth_token_fail(decorators.auth_token):
-#             def target(self, *args, **kwargs):
-#                 raise ValueError("failed auth_token_fail.target")
-# 
-#         class auth_child(decorators.auth):
-#             def target(self, *args, **kwargs):
-#                 try:
-#                     auth = auth_token_fail()
-#                     auth.target(*args, **kwargs)
-# 
-#                 except ValueError as e:
-#                     pout.v(e)
-# #                     if e.code == 401:
-# #                         auth = auth_another()
-# #                         auth.target(self.request, args, kwargs)
-# 
-#                 return True
-# 
-# 
-# 
-#                 #pout.v(args, kwargs)
-#                 #raise ValueError()
-# 
-#         class ACController(object):
-#             @auth_child
-#             def foo(self): pass
-# 
-#         c = ACController()
-#         c.request = endpoints.Request()
-#         c.foo()
-
-
-class PropertyTest(TestCase):
-    def test__property_init(self):
-        counts = dict(fget=0, fset=0, fdel=0)
-        def fget(self):
-            counts["fget"] += 1
-            return self._v
-
-        def fset(self, v):
-            counts["fset"] += 1
-            self._v = v
-
-        def fdel(self):
-            counts["fdel"] += 1
-            del self._v
-
-        class FooPropInit(object):
-            v = endpoints.decorators._property(fget, fset, fdel, "this is v")
-        f = FooPropInit()
-        f.v = 6
-        self.assertEqual(6, f.v)
-        self.assertEqual(2, sum(counts.values()))
-        del f.v
-        self.assertEqual(3, sum(counts.values()))
-
-        counts = dict(fget=0, fset=0, fdel=0)
-        class FooPropInit2(object):
-            v = endpoints.decorators._property(fget=fget, fset=fset, fdel=fdel, doc="this is v")
-        f = FooPropInit2()
-        f.v = 6
-        self.assertEqual(6, f.v)
-        self.assertEqual(2, sum(counts.values()))
-        del f.v
-        self.assertEqual(3, sum(counts.values()))
-
-    def test__property_allow_empty(self):
-        class PAE(object):
-            foo_val = None
-            @endpoints.decorators._property(allow_empty=False)
-            def foo(self):
-                return self.foo_val
-
-        c = PAE()
-        self.assertEqual(None, c.foo)
-        self.assertFalse('_foo' in c.__dict__)
-
-        c.foo_val = 1
-        self.assertEqual(1, c.foo)
-        self.assertTrue('_foo' in c.__dict__)
-
-    def test__property_setter(self):
-        class WPS(object):
-            foo_get = False
-            foo_set = False
-            foo_del = False
-
-            @endpoints.decorators._property
-            def foo(self):
-                self.foo_get = True
-                return 1
-
-            @foo.setter
-            def foo(self, val):
-                self.foo_set = True
-                self._foo = val
-
-            @foo.deleter
-            def foo(self):
-                self.foo_del = True
-                del(self._foo)
-
-        c = WPS()
-
-        self.assertEqual(1, c.foo)
-
-        c.foo = 5
-        self.assertEqual(5, c.foo)
-
-        del(c.foo)
-        self.assertEqual(1, c.foo)
-
-        self.assertTrue(c.foo_get)
-        self.assertTrue(c.foo_set)
-        self.assertTrue(c.foo_del)
-
-    def test__property__strange_behavior(self):
-        class BaseFoo(object):
-            def __init__(self):
-                setattr(self, 'bar', None)
-
-            def __setattr__(self, n, v):
-                super(BaseFoo, self).__setattr__(n, v)
-
-        class Foo(BaseFoo):
-            @endpoints.decorators._property(allow_empty=False)
-            def bar(self):
-                return 1
-
-        f = Foo()
-        self.assertEqual(1, f.bar)
-
-        f.bar = 2
-        self.assertEqual(2, f.bar)
-
-    def test__property___dict__direct(self):
-        """
-        this is a no win situation
-
-        if you have a bar _property and a __setattr__ that modifies directly then
-        the other _property values like __set__ will not get called, and you can't
-        have _property.__get__ look for the original name because there are times
-        when you want your _property to override a parent's original value for the
-        property, so I've chosen to just ignore this case and not support it
-        """
-        class Foo(object):
-            @endpoints.decorators._property
-            def bar(self):
-                return 1
-            def __setattr__(self, field_name, field_val):
-                self.__dict__[field_name] = field_val
-                #super(Foo, self).__setattr__(field_name, field_val)
-
-        f = Foo()
-        f.bar = 2 # this will be ignored
-        self.assertEqual(1, f.bar)
-
-    def test__property(self):
-        class WP(object):
-            count_foo = 0
-
-            @endpoints.decorators._property(True)
-            def foo(self):
-                self.count_foo += 1
-                return 1
-
-            @endpoints.decorators._property(read_only=True)
-            def baz(self):
-                return 2
-
-            @endpoints.decorators._property()
-            def bar(self):
-                return 3
-
-            @endpoints.decorators._property
-            def che(self):
-                return 4
-
-        c = WP()
-        r = c.foo
-        self.assertEqual(1, r)
-        self.assertEqual(1, c._foo)
-        with self.assertRaises(AttributeError):
-            c.foo = 2
-        with self.assertRaises(AttributeError):
-            del(c.foo)
-        c.foo
-        c.foo
-        self.assertEqual(1, c.count_foo)
-
-        r = c.baz
-        self.assertEqual(2, r)
-        self.assertEqual(2, c._baz)
-        with self.assertRaises(AttributeError):
-            c.baz = 3
-        with self.assertRaises(AttributeError):
-            del(c.baz)
-
-        r = c.bar
-        self.assertEqual(3, r)
-        self.assertEqual(3, c._bar)
-        c.bar = 4
-        self.assertEqual(4, c.bar)
-        self.assertEqual(4, c._bar)
-        del(c.bar)
-        r = c.bar
-        self.assertEqual(3, r)
-
-        r = c.che
-        self.assertEqual(4, r)
-        self.assertEqual(4, c._che)
-        c.che = 4
-        self.assertEqual(4, c.che)
-        del(c.che)
-        r = c.che
-        self.assertEqual(4, r)
-
 
 class ParamTest(TestCase):
     def test_type_issue_76(self):
@@ -1188,6 +969,32 @@ class ParamTest(TestCase):
 
 
 class RouteTest(TestCase):
+    def test_issue94(self):
+        """https://github.com/Jaymon/endpoints/issues/94"""
+        c = Server(contents=[
+            "from endpoints import Controller",
+            "from endpoints.decorators import route, param",
+            "class Default(Controller):",
+            "    @route(lambda req: 'foo' in req.kwargs)",
+            "    @param('foo')",
+            "    def POST_foo(self, **kwargs):",
+            "        return 'foo'",
+            "",
+            "    @route(lambda req: 'foo' not in req.kwargs)",
+            "    @param('bar')",
+            "    @param('che')",
+            "    def POST_no_foo(self, **kwargs):",
+            "        return 'no foo'",
+        ])
+
+        res = c.handle("/", method="POST", body_kwargs={"bar": 1, "che": 2})
+        self.assertEqual(200, res.code)
+        self.assertEqual("no foo", res.body)
+
+        res = c.handle("/", method="POST", body_kwargs={"foo": 1})
+        self.assertEqual(200, res.code)
+        self.assertEqual("foo", res.body)
+
     def test_error(self):
         c = Server(contents=[
             "from endpoints import Controller",
@@ -1226,10 +1033,10 @@ class RouteTest(TestCase):
         self.assertEqual("bar", res.body)
 
         res = c.handle("/foo")
-        self.assertEqual(405, res.code)
+        self.assertEqual(404, res.code)
 
         res = c.handle("/foo/baz")
-        self.assertEqual(405, res.code)
+        self.assertEqual(404, res.code)
 
     def test_param_route_keys(self):
         c = Server(contents=[
@@ -1350,20 +1157,21 @@ class RouteTest(TestCase):
             self.assertEqual(2, res._body)
 
         res = c.handle("/foo1/2")
-        self.assertEqual(405, res.code)
+        self.assertEqual(404, res.code)
 
         for path in ["/foo3/2/3", "/foo4/2/3"]:
             res = c.handle(path)
             self.assertEqual(3, res._body)
 
         res = c.handle("/foo1/2/3")
-        self.assertEqual(405, res.code)
+        self.assertEqual(404, res.code)
+
         res = c.handle("/foo2/2/3")
         self.assertEqual(405, res.code)
 
         for path in ["/foo1/2/3/4", "/foo2/2/3/4", "/foo3/2/3/4"]:
             res = c.handle(path)
-            self.assertEqual(405, res.code)
+            self.assertTrue(res.code in set([404, 405]))
 
         res = c.handle("/foo4/2/3/4")
         self.assertEqual(4, res._body)
@@ -1450,11 +1258,10 @@ class VersionTest(TestCase):
         self.assertEqual(2, res._body)
 
         res = c.handle("/foo", version="v3")
-        self.assertEqual(405, res.code)
+        self.assertEqual(404, res.code)
 
     def test_complex(self):
-        controller_prefix = "version_complex"
-        c = Server(controller_prefix, [
+        c = Server(contents=[
             "from endpoints import Controller",
             "from endpoints.decorators import version, route",
             "",
@@ -1493,8 +1300,7 @@ class VersionTest(TestCase):
 
 class CodeErrorTest(TestCase):
     def test_raise(self):
-        controller_prefix = "ce_raise"
-        c = Server(controller_prefix, [
+        c = Server(contents=[
             "from endpoints import Controller",
             "from endpoints.decorators import code_error, param",
             "",
